/*
 * NewMadeleine
 * Copyright (C) 2014-2015 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <mpi.h>
#include <pthread.h>
#include "mpi_bench_generic.h"

#define COUNT 100

static int threads = 0;
static struct
{
  int iterations;
  size_t len;
  int dest;
} global = { .iterations = -1, .len = 0, .dest = 0 };

static struct mpi_bench_param_bounds_s param_bounds =
  {
    .min  = 1,
    .max  = THREADS_DEFAULT,
    .mult = 1,
    .incr = 1
  };

static pthread_t tids[THREADS_MAX];


static void*ping_thread(void*_i)
{
  const int i = (uintptr_t)_i; /* thread id */
  int stop = 0;
  size_t len = global.len;
  void*buf = malloc(len);
  while(!stop)
    {
      if(mpi_bench_common.is_server)
	{
	  MPI_Recv(buf, global.len, MPI_CHAR, mpi_bench_common.peer, TAG + i, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
	  MPI_Send(buf, 0, MPI_CHAR, mpi_bench_common.peer, TAG + i, MPI_COMM_WORLD);
	}
      else
	{
	  MPI_Send(buf, 0, MPI_CHAR, mpi_bench_common.peer, TAG + i, MPI_COMM_WORLD);
	}
    }
  free(buf);
  return NULL;
}

static const struct mpi_bench_param_bounds_s*mpi_bench_thread_NtoN_getparams(void)
{
  param_bounds.max = mpi_bench_get_threads();
  return &param_bounds;
}

static void mpi_bench_thread_NtoN_setparam(int param)
{
  threads = param;
}

static void mpi_bench_thread_NtoN_endparam(void)
{
  threads = 0;
}

static void mpi_bench_thread_NtoN_init(void*buf, size_t len, int count)
{
  global.iterations = count;
  global.len = len;
  global.dest = 0;
  if(mpi_bench_common.is_server)
    {
      /* start this round threads */
      int i;
      for(i = 0; i < threads; i++)
	{
	  pthread_create(&tids[i], NULL, &ping_thread, (void*)(uintptr_t)i);
	}
    }
}

static void mpi_bench_thread_NtoN_finalize(void)
{
  if(mpi_bench_common.is_server)
    {
      /* stop previous threads */
      int i;
      for(i = 0; i < threads; i++)
	{
	  pthread_join(tids[i], NULL);
	}
    }
}

static void mpi_bench_thread_NtoN_server(void*buf, size_t len)
{
  /* do nothing- work is done in thread */
}

static void mpi_bench_thread_NtoN_client(void*buf, size_t len)
{
  MPI_Send(buf, len, MPI_CHAR, mpi_bench_common.peer, TAG + global.dest, MPI_COMM_WORLD);
  MPI_Recv(buf, 0, MPI_CHAR, mpi_bench_common.peer, TAG + global.dest, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
  global.dest = (global.dest + 1) % threads;
}


const struct mpi_bench_s mpi_bench_thread_NtoN =
  {
    .label      = "mpi_bench_thread_NtoN",
    .name       = "MPI threaded N to N threads",
    .rtt        = 0,
    .threads    = 1,
    .server     = &mpi_bench_thread_NtoN_server,
    .client     = &mpi_bench_thread_NtoN_client,
    .getparams  = &mpi_bench_thread_NtoN_getparams,
    .setparam   = &mpi_bench_thread_NtoN_setparam,
    .endparam   = &mpi_bench_thread_NtoN_endparam,
    .init       = &mpi_bench_thread_NtoN_init,
    .finalize   = &mpi_bench_thread_NtoN_finalize
  };

